<?php

namespace Drupal\file_temporary_validator\Service;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Drupal\file\FileInterface;

/**
 * Class File Temporary Validator.
 *
 * @package file_temporary_validator
 */
final class FileTemporaryValidator {
  use StringTranslationTrait;

  /**
   * Constructs the file temporary validator service.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\Core\Session\AccountInterface $currentUser
   *   The current user.
   */
  public function __construct(
    private readonly EntityTypeManagerInterface $entityTypeManager,
    private readonly AccountInterface $currentUser,
  ) {}

  /**
   * Check Temporary Files function.
   *
   * @param \Drupal\file\Entity\File $file
   *   File.
   *
   * @return array
   *   Return Response Array.
   */
  public function checkTemporaryFileViolation(FileInterface $file): array {
    if ($duplicate = $this->getFileDuplications($file)) {
      return [$this->buildDuplicateViolation($file, $duplicate)];
    }
    return [];
  }

  /**
   * Queries and retrieves any temporary file duplications.
   *
   * @param \Drupal\file\FileInterface $file
   *   File.
   *
   * @return \Drupal\file\FileInterface|null
   *   Return the file if found.
   */
  public function getFileDuplications(FileInterface $file): ?FileInterface {
    $storage = $this->entityTypeManager->getStorage('file');

    $query = $storage->getQuery()
      ->accessCheck(FALSE)
      ->condition('status', FileInterface::STATUS_PERMANENT, '<>')
      ->condition('filename', $file->getFilename());

    if ($file->id()) {
      $query->condition('fid', $file->id(), '<>');
    }

    $fids = $query->range(0, 1)->execute();
    return $fids ? $storage->load(reset($fids)) : NULL;
  }

  /**
   * Builds message parameters for a duplicate file violation message.
   */
  public function buildDuplicateViolation(FileInterface $current, FileInterface $duplicate): TranslatableMarkup {
    $parameters = [
      '@file_location' => $current->getFileUri(),
      '@file_name' => $duplicate->label(),
    ];

    if ($this->currentUser->hasPermission('access files overview')) {
      $parameters[':duplicate_url'] = Url::fromRoute(
        'view.files.page_1',
        ['filename' => $duplicate->getFilename()],
        ['attributes' => ['target' => '_blank']],
      )->toString();

      return new TranslatableMarkup(
        'Error while uploading file: Temporary file <a href=":duplicate_url" target="_blank">@file_name</a> already exists at @file_location.',
        $parameters,
      );
    }

    return new TranslatableMarkup(
      'Error while uploading file: Temporary file @file_name already exists at @file_location.',
      $parameters,
    );
  }

}
