<?php

declare(strict_types=1);

namespace Drupal\Tests\file_temporary_validator\Functional;

use Drupal\file\Entity\File;

/**
 * Tests that duplicate temporary files are blocked on the media form.
 *
 * @group file_temporary_validator
 */
final class FileTemporaryValidatorDocumentTest extends FileTemporaryValidatorTestBase {

  /**
   * Enabling the validator and testing duplicate file upload rejection.
   */
  public function testDuplicateTemporaryFileOnMediaAdd(): void {
    $user = $this->drupalCreateUser([
      'access media overview',
      'view media',
      'create document media',
      'administer media',
      'administer media form display',
      'administer media fields',
      'administer media types',
      'access content',
    ]);
    $this->drupalLogin($user);

    // Go to the field-config edit form for the document media file field.
    $this->drupalGet('/admin/structure/media/manage/document/fields/media.document.field_media_file');
    $this->assertSession()->statusCodeEquals(200);

    // Confirm the checkbox exists and is unchecked by default.
    $this->assertSession()->fieldExists('third_party_settings[file_temporary_validator][enabled]');
    $this->assertSession()->checkboxNotChecked('third_party_settings[file_temporary_validator][enabled]');

    // Enable it and save.
    $page = $this->getSession()->getPage();
    $page->checkField('third_party_settings[file_temporary_validator][enabled]');
    $page->pressButton('Save');

    // Go to the media-add form.
    $this->drupalGet('/media/add/document');

    // Prepare a file and seed it as a temporary file entity.
    $file_1 = ['uri' => 'temporary://file_1.txt', 'data' => 'file 1 original'];
    file_put_contents($file_1['uri'], $file_1['data']);
    $file_entity = File::create(['uri' => $file_1['uri'], 'status' => 0]);
    $file_entity->save();

    // Attempt to upload that same file.
    $page->fillField('Name', 'Foobar');
    $page->attachFileToField('File', \Drupal::service('file_system')->realpath($file_1['uri']));
    $page->pressButton('Save');

    // Verify that the validator blocked it.
    $this->assertSession()->pageTextContains('Error while uploading file');
    $this->assertSession()->pageTextMatches('/file_1\.txt/');
    $this->assertSession()->addressEquals('/media/add/document');

    unlink($file_1['uri']);

    // Now upload a different file with a different name.
    $file_2 = ['uri' => 'temporary://file_2.txt', 'data' => 'file 2 content'];
    file_put_contents($file_2['uri'], $file_2['data']);

    // Attempt to upload the new file.
    $page->fillField('Name', 'Unique');
    $page->attachFileToField('File', \Drupal::service('file_system')->realpath($file_2['uri']));
    $page->pressButton('Save');

    // Verify that the new file was uploaded successfully.
    $this->assertSession()->pageTextContains('Document Unique has been created.');

    unlink($file_2['uri']);
  }

}
